"""
OpenMemo Feedback Aggregator

Bridge layer that transforms raw Feedback Loop signals into
Inspector-ready structured insights.

PRD §15 (one-line summary):
"Feedback Aggregator turns 'memory behavior data' into
 'results humans can understand'."

    from openmemo.feedback_aggregator import FeedbackAggregator
    from openmemo.memory_feedback import FeedbackStore

    store = FeedbackStore("openmemo.db")
    agg = FeedbackAggregator(store)

    # PRD §4.1 — Task chain trace
    trace = agg.task_trace("task_001")

    # PRD §4.2 — Memory quality profile
    profile = agg.memory_profile("mem_123")

    # PRD §4.3 — System impact metrics
    metrics = agg.impact_metrics()

    # PRD §4.4 — Chronological event timeline
    events = agg.timeline("task_001")
"""

from openmemo.feedback_aggregator.aggregator import FeedbackAggregator
from openmemo.feedback_aggregator.task_trace_builder import TaskTraceBuilder
from openmemo.feedback_aggregator.memory_profile_builder import MemoryProfileBuilder
from openmemo.feedback_aggregator.metrics_builder import MetricsBuilder
from openmemo.feedback_aggregator.timeline_builder import TimelineBuilder
from openmemo.feedback_aggregator.utils import to_iso, safe_div


__all__ = [
    "FeedbackAggregator",
    "TaskTraceBuilder",
    "MemoryProfileBuilder",
    "MetricsBuilder",
    "TimelineBuilder",
    "to_iso",
    "safe_div",
]
